#include "wrappers.h"
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <unistd.h>
#include <signal.h>
#include <err.h>
#include <stdlib.h>


size_t readBackend(int fd, char * buf, size_t maxlen)
{
    int len = read(fd, buf, maxlen);
    if (len < 0) {
        len = 0;
    }

    return len;
}

int stopBackend(int pid, int pipefd)
{
    close(pipefd);
    kill(pid, SIGTERM);
    int status;
    wait(&status);
    return status;
}

int startBackend(const char * backend, int * pipefd)
{
    /* Create a pipe */
    int pipe_fds[2];
    if (pipe(pipe_fds)) {
        warn("Failed to open pipe, cannot start backend!");
        return -1;
    }

    int pid = fork();
    if (!pid) {
        /* Child closes read end of pipe */
        close(pipe_fds[0]);

        write(pipe_fds[1], "From child", 11);
        /* dup write end of pipe to stdout (FD 1), closing old stdout */
        //dup2(pipe_fds[1], 1);
        //dup2(pipe_fds[1], 2);

        write(2, "Child stderr\n", 13);
        printf("Child stdout\n");

        /* Don't need whatever FD is in pipe_fds[1] anymore, since it is stdout now */
        close(pipe_fds[1]);

        execl(backend, "BackEnd", NULL);
        exit(0);
    }

    /* Return the read end of the pipe in pipefd pointer */
    *pipefd = pipe_fds[0];


    /* Parent closes write end of the pipe */
    close(pipe_fds[1]);


    return pid;
}
